import traceback

from cpl_core.configuration.configuration_model_abc import ConfigurationModelABC
from cpl_core.console import Console


class ServerSettings(ConfigurationModelABC):

    def __init__(self):
        ConfigurationModelABC.__init__(self)

        self._id: int = 0
        self._message_delete_timer: int = 0
        self._login_message_channel_id: int = 0
        self._login_message: str = ''
        self._welcome_message: str = ''
        self._goodbye_message: str = ''
        self._max_voice_state_hours: int = 0
        self._xp_per_message: int = 0
        self._xp_per_ontime_hour: int = 0
        self._afk_channel_ids: list[int] = []

    @property
    def id(self) -> str:
        return self._id
    
    @property
    def message_delete_timer(self) -> int:
        return self._message_delete_timer
    
    @property
    def login_message_channel_id(self) -> int:
        return self._login_message_channel_id
    
    @property
    def login_message(self) -> str:
        return self._login_message

    @property
    def welcome_message(self) -> str:
        return self._welcome_message

    @property
    def goodbye_message(self) -> str:
        return self._goodbye_message

    @property
    def max_voice_state_hours(self) -> int:
        return self._max_voice_state_hours

    @property
    def xp_per_message(self) -> int:
        return self._xp_per_message

    @property
    def xp_per_ontime_hour(self) -> int:
        return self._xp_per_ontime_hour

    @property
    def afk_channel_ids(self) -> list[int]:
        return self._afk_channel_ids

    def from_dict(self, settings: dict):
        try:
            self._id = int(settings['Id'])
            self._message_delete_timer = int(settings['MessageDeleteTimer'])
            self._login_message_channel_id = int(settings['LoginMessageChannelId'])
            self._login_message = settings['LoginMessage']
            self._welcome_message = settings['WelcomeMessage']
            self._goodbye_message = settings['GoodbyeMessage']
            self._max_voice_state_hours = int(settings['MaxVoiceStateHours'])
            self._xp_per_message = int(settings['XpPerMessage'])
            self._xp_per_ontime_hour = int(settings['XpPerOntimeHour'])
            for id in settings['AFKChannelIds']:
                self._afk_channel_ids.append(int(id))
        except Exception as e:
            Console.error(f'[ ERROR ] [ {__name__} ]: Reading error in settings')
            Console.error(f'[ EXCEPTION ] [ {__name__} ]: {e} -> {traceback.format_exc()}')
